@@ -13,7 +13,6 @@ del get_versions
 
 
 import itertools
-import operator
 
 import numpy
 import scipy.ndimage
@@ -379,41 +378,15 @@ def maximum_position(input, labels=None, index=None):
     if index.shape:
         index = index.flatten()
 
-    indices = _utils._ravel_shape_indices(
-        input.shape, chunks=input.chunks
+    max_1dpos_lbl = labeled_comprehension(
+        input, labels, index, _utils._argmax, int, 0, pass_positions=True
     )
 
-    max_lbl = maximum(input, labels=labels, index=index)
-
-    lbl_mtch = _utils._get_label_matches(labels, index)
-    input_mtch = dask.array.where(
-        lbl_mtch, input[index.ndim * (None,)], numpy.nan
-    )
-
-    max_lbl_mask = operator.eq(
-        max_lbl[max_lbl.ndim * (slice(None),) + input.ndim * (None,)],
-        input_mtch
-    )
-    max_lbl_mask_any = max_lbl_mask.any(
-        axis=tuple(_pycompat.irange(index.ndim, max_lbl_mask.ndim))
-    )
-    max_lbl_indices = dask.array.where(
-        max_lbl_mask, indices, numpy.nan
-    )
-
-    max_1dpos_lbl = dask.array.where(
-        max_lbl_mask_any,
-        dask.array.nanmin(
-            max_lbl_indices,
-            axis=tuple(_pycompat.irange(index.ndim, max_lbl_indices.ndim))
-        ),
-        0
-    ).astype(indices.dtype)
+    if not max_1dpos_lbl.ndim:
+        max_1dpos_lbl = max_1dpos_lbl[None]
 
     max_pos_lbl = []
     max_1dpos_lbl_rem = max_1dpos_lbl
-    if not max_1dpos_lbl_rem.ndim:
-        max_1dpos_lbl_rem = max_1dpos_lbl_rem[None]
     for i in _pycompat.irange(input.ndim):
         d = int(numpy.prod(input.shape[i + 1:]))
         max_pos_lbl.append(max_1dpos_lbl_rem // d)
@@ -428,7 +401,7 @@ def maximum_position(input, labels=None, index=None):
 
 def mean(input, labels=None, index=None):
     """
-    Calculate the mean of the values of an array at labels.
+    Find the mean over an image at specified subregions.
 
     Parameters
     ----------
@@ -560,41 +533,15 @@ def minimum_position(input, labels=None, index=None):
     if index.shape:
         index = index.flatten()
 
-    indices = _utils._ravel_shape_indices(
-        input.shape, chunks=input.chunks
+    min_1dpos_lbl = labeled_comprehension(
+        input, labels, index, _utils._argmin, int, 0, pass_positions=True
     )
 
-    min_lbl = minimum(input, labels=labels, index=index)
-
-    lbl_mtch = _utils._get_label_matches(labels, index)
-    input_mtch = dask.array.where(
-        lbl_mtch, input[index.ndim * (None,)], numpy.nan
-    )
-
-    min_lbl_mask = operator.eq(
-        min_lbl[min_lbl.ndim * (slice(None),) + input.ndim * (None,)],
-        input_mtch
-    )
-    min_lbl_mask_any = min_lbl_mask.any(
-        axis=tuple(_pycompat.irange(index.ndim, min_lbl_mask.ndim))
-    )
-    min_lbl_indices = dask.array.where(
-        min_lbl_mask, indices, numpy.nan
-    )
-
-    min_1dpos_lbl = dask.array.where(
-        min_lbl_mask_any,
-        dask.array.nanmin(
-            min_lbl_indices,
-            axis=tuple(_pycompat.irange(index.ndim, min_lbl_indices.ndim))
-        ),
-        0
-    ).astype(indices.dtype)
+    if not min_1dpos_lbl.ndim:
+        min_1dpos_lbl = min_1dpos_lbl[None]
 
     min_pos_lbl = []
     min_1dpos_lbl_rem = min_1dpos_lbl
-    if not min_1dpos_lbl_rem.ndim:
-        min_1dpos_lbl_rem = min_1dpos_lbl_rem[None]
     for i in _pycompat.irange(input.ndim):
         d = int(numpy.prod(input.shape[i + 1:]))
         min_pos_lbl.append(min_1dpos_lbl_rem // d)
@@ -641,7 +588,7 @@ def standard_deviation(input, labels=None, index=None):
 
 def sum(input, labels=None, index=None):
     """
-    Calculate the sum of the values of the array.
+    Find the sum over an image at specified subregions.
 
     Parameters
     ----------
@@ -665,16 +612,8 @@ def sum(input, labels=None, index=None):
         input, labels, index
     )
 
-    lbl_mtch = _utils._get_label_matches(labels, index)
-
-    input_mtch = dask.array.where(
-        lbl_mtch, input[index.ndim * (None,)], input.dtype.type(0)
-    )
-
-    input_mtch = input_mtch.astype(numpy.float64)
-
-    sum_lbl = input_mtch.sum(
-        axis=tuple(_pycompat.irange(index.ndim, input_mtch.ndim))
+    sum_lbl = labeled_comprehension(
+        input, labels, index, numpy.sum, numpy.float64, numpy.float64(0)
     )
 
     return sum_lbl
